package com.hz.pm.api.open.manage;

import cn.hutool.core.util.NumberUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.ExistsSqlConst;
import com.hz.pm.api.common.statemachine.event.ProjectStateChangeEvent;
import com.hz.pm.api.common.util.BizDateUtil;
import com.hz.pm.api.open.model.po.MhSyncProjectListPO;
import com.hz.pm.api.open.model.vo.DeclaredProjectInfoToMhVO;
import com.hz.pm.api.open.model.vo.ReferSystemReplaceInfoToMhVO;
import com.hz.pm.api.open.util.OpenApiCheckSignUtil;
import com.hz.pm.api.projectlib.entity.MhSystemReplaceInfo;
import com.hz.pm.api.projectlib.entity.ProjectGovSystemReplaceInfos;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.GovSystemReplaceTypeEnum;
import com.hz.pm.api.projectlib.service.IMhSystemReplaceInfoService;
import com.hz.pm.api.projectlib.service.IProjectGovSystemReplaceInfosService;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * <p>
 * OpenApiProjectLibManage
 * </p>
 *
 * @author WendyYang
 * @since 10:22 2024/11/7
 */
@Component
@RequiredArgsConstructor
public class OpenApiProjectLibManage {

    private final OpenApiCheckSignUtil openApiCheckSignUtil;
    private final IProjectService projectService;
    private final IProjectGovSystemReplaceInfosService systemReplaceInfosService;
    private final IMhSystemReplaceInfoService mhSystemReplaceInfoService;

    /**
     * 获取已复核通过的项目列表
     *
     * @param po 参数
     * @return 复核通过的项目
     */
    public List<DeclaredProjectInfoToMhVO> listDeclaredProjects(MhSyncProjectListPO po) {
        openApiCheckSignUtil.checkSign(po.getKey());
        StringBuilder sql = new StringBuilder(ExistsSqlConst.PROJECT_EXISTS_STATUS_CHANGE)
                .append(StrUtil.format(" and npsc.event = '{}'", ProjectStateChangeEvent.PROJECT_APPLICATION_SUBMIT));
        if (po.getCreateOn() != null) {
            sql.append(String.format(" and npsc.create_on >= '%s'", BizDateUtil.toDateStr(po.getCreateOn())));
        }
        Wrapper<Project> query = Wrappers.lambdaQuery(Project.class)
                .select(Project::getId, Project::getProjectCode, Project::getProjectName, Project::getBuildOrgCode,
                        Project::getStatus, Project::getReviewAmount, Project::getDeclareAmount, Project::getProjectYear,
                        Project::getCreateOn, Project::getBuildOrgName)
                .eq(Project::getNewest, Boolean.TRUE)
                .exists(sql.toString())
                .eq(po.getProjectYear() != null, Project::getProjectYear, po.getProjectYear())
                .orderByDesc(Project::getCreateOn);
        List<Project> projects = projectService.list(query);
        if (projects.isEmpty()) {
            return Collections.emptyList();
        }
        List<String> projectCodes = CollUtils.fieldList(projects, Project::getProjectCode);
        List<ProjectGovSystemReplaceInfos> replaceInfos = systemReplaceInfosService.listByProjectCodes(projectCodes);
        Map<String, List<ProjectGovSystemReplaceInfos>> originReplaceInfoMap;
        Map<String, List<MhSystemReplaceInfo>> replaceInfoMap;
        if (!replaceInfos.isEmpty()) {
            originReplaceInfoMap = CollUtils.group(replaceInfos, ProjectGovSystemReplaceInfos::getProjectCode);
            List<String> sourceSystemIds = CollUtils.fieldList(replaceInfos, ProjectGovSystemReplaceInfos::getSourceSystemId);
            List<MhSystemReplaceInfo> mhSystemReplaceInfos = mhSystemReplaceInfoService.listByIds(sourceSystemIds);
            Map<String, MhSystemReplaceInfo> mhReplaceInfoMap = CollUtils.listToMap(mhSystemReplaceInfos, MhSystemReplaceInfo::getId);
            replaceInfoMap = replaceInfos.stream()
                    .filter(w -> mhReplaceInfoMap.containsKey(w.getSourceSystemId()))
                    .collect(Collectors.groupingBy(ProjectGovSystemReplaceInfos::getProjectCode,
                            Collectors.mapping(w -> mhReplaceInfoMap.get(w.getSourceSystemId()), Collectors.toList())));
        } else {
            originReplaceInfoMap = Collections.emptyMap();
            replaceInfoMap = Collections.emptyMap();
        }

        return projects.stream().map(w -> {
            DeclaredProjectInfoToMhVO item = new DeclaredProjectInfoToMhVO();
            item.setId(w.getId());
            item.setStatus(w.getStatus());
            item.setCreateOn(w.getCreateOn());
            item.setProjectYear(w.getProjectYear());
            item.setProjectCode(w.getProjectCode());
            item.setProjectName(w.getProjectName());
            item.setBuildOrgCode(w.getBuildOrgCode());
            item.setBuildOrgName(w.getBuildOrgName());
            item.setDeclareAmount(w.getDeclareAmount());
            item.setReviewAmount(w.getReviewAmount());
            if (replaceInfoMap.containsKey(w.getProjectCode())) {
                List<MhSystemReplaceInfo> currReplaceInfos = replaceInfoMap.get(w.getProjectCode());
                List<ReferSystemReplaceInfoToMhVO> referSystemList = CollUtils.convert(currReplaceInfos,
                        t -> ReferSystemReplaceInfoToMhVO.builder()
                                .id(t.getId())
                                .buildOrgCode(t.getBuildOrgCode())
                                .buildOrgName(t.getBuildOrgName())
                                .systemName(t.getSourceName())
                                .alternativeName(t.getTargetName())
                                .alternativeType(t.getReplaceType())
                                .build());
                item.setReferSystemList(referSystemList);
            } else if (originReplaceInfoMap.containsKey(w.getProjectCode())) {
                List<ProjectGovSystemReplaceInfos> currReplaceInfos = originReplaceInfoMap.get(w.getProjectCode());
                List<ReferSystemReplaceInfoToMhVO> referSystemList = CollUtils.convert(currReplaceInfos,
                        t -> ReferSystemReplaceInfoToMhVO.builder()
                                .id(t.getSourceSystemId())
                                .buildOrgCode(Integer.valueOf(w.getBuildOrgCode()))
                                .buildOrgName(w.getBuildOrgName())
                                .systemName(t.getSourceSystem())
                                .alternativeName(t.getTargetSystem())
                                .alternativeType(getReplaceType(t))
                                .build());
                item.setReferSystemList(referSystemList);
            } else {
                item.setReferSystemList(Collections.emptyList());
            }
            return item;
        }).collect(Collectors.toList());
    }

    private static String getReplaceType(ProjectGovSystemReplaceInfos t) {
        String replaceType = StrUtil.EMPTY;
        if (NumberUtil.isNumber(t.getReplaceType())) {
            Optional<GovSystemReplaceTypeEnum> optReplaceType = GovSystemReplaceTypeEnum.get(Integer.valueOf(t.getReplaceType()));
            if (optReplaceType.isPresent()) {
                GovSystemReplaceTypeEnum replaceTypeEnum = optReplaceType.get();
                replaceType = Optional.ofNullable(replaceTypeEnum.getMhReplaceType())
                        .flatMap(mhReplaceType -> Optional.of(mhReplaceType.getCode()))
                        .orElse(replaceTypeEnum.getVal());
            }
        }
        return replaceType;
    }

}
